import 'dart:async';
import 'dart:ui';

import 'package:flutter/services.dart';

class YinWebview {
  static MethodChannel _channel;

  YinWebview() {
    _channel = const MethodChannel('yin_webview');
  }

  ///开启
  Future<Null> launch(
    String url, {
    Rect rect,
    Map<String, String> headers,
    bool withJavascript,
    bool clearCache,
    bool clearCookies,
    List<Map<String, String>> cookieList,
    bool hidden,
    bool enableAppScheme,
    String userAgent,
    bool withZoom,
    bool withLocalStorage,
    bool withLocalUrl,
    bool scrollBar,
    bool allowFileURLs,
    Function webTitle,
    Function loadUrl,
  }) async {
    final args = <String, dynamic>{
      'url': url,
      'withJavascript': withJavascript ?? true,
      'clearCookies': clearCookies ?? false,
      'cookieList': cookieList ?? [],
      'enableAppScheme': enableAppScheme ?? true,
      'userAgent': userAgent,
      'withZoom': withZoom ?? false,
      'withLocalStorage': withLocalStorage ?? true,
      'withLocalUrl': withLocalUrl ?? false,
      'scrollBar': scrollBar ?? true,
      'allowFileURLs': allowFileURLs ?? false,
    };

    if (headers != null) {
      args['headers'] = headers;
    }

    if (rect != null) {
      args["rect"] = {"left": rect.left, "top": rect.top, "width": rect.width, "height": rect.height};
    }

    ///原生回调
    _channel.setMethodCallHandler((handler) {
      String method = handler.method;
      if (method == "title" && webTitle != null) {
        webTitle(handler.arguments.toString());
      } else if (method == "url" && loadUrl != null) {
        loadUrl(handler.arguments.toString());
      }
    });

    await _channel.invokeMethod('launch', args);
  }

  ///关闭
  Future<Null> close() async => await _channel.invokeMethod('close');

  ///重新加载
  Future<Null> reload() async => await _channel.invokeMethod('reload');

  ///后退
  Future<Null> back() async => await _channel.invokeMethod('back');

  ///向前
  Future<Null> forward() async => await _channel.invokeMethod('forward');

  ///清理cookie
  Future<Null> cleanCookies() async => await _channel.invokeMethod('cleanCookies');

  ///获取cookies
  Future<Null> getCookies(String url, Function getCookie) async {
    final args = {'url': url};
    String cookies = await _channel.invokeMethod('getAllCookies', args);
    getCookie(cookies);
  }

  /// 设置大小
  Future<Null> resize(Rect rect) async {
    final args = {};
    args['rect'] = {
      'left': rect.left,
      'top': rect.top,
      'width': rect.width,
      'height': rect.height,
    };
    await _channel.invokeMethod('resize', args);
  }

  ///销毁
  void dispose() {
    _channel = null;
  }
}
